@@ -1,10 +1,12 @@
 #' Load GWAS results into a single data.frame
 #'
+#' A1 is the reference allele and A2 is the alternate allele.
+#' 
 #' Two columns are added, \code{Z} and \code{P}. \code{Z} is the
 #' focal parameter divded by its standard error. \code{P} is the
 #' unadjusted two-sided normal CDF corresponding to the absolute
 #' \code{Z} score.
-#' 
+#'
 #' @param path vector of paths to result files created by \link{GWAS}
 #' @param focus parameter name on which to calculate a Z score and p-value
 #' @param extraColumns character vector of additional columns to load
@@ -17,13 +19,13 @@
 #' tdir <- tempdir()
 #' dir <- system.file("extdata", package = "gwsem")
 #' pheno <- data.frame(anxiety=rnorm(500))
-#' m1 <- buildOneItem(pheno, 'anxiety')
+#' m1 <- buildItem(pheno, 'anxiety')
 #' GWAS(m1, file.path(dir,"example.pgen"),
 #'     file.path(tdir,"out.log"))
 #' loadResults(file.path(tdir,"out.log"), "snp2anxiety")
 loadResults <- function(path, focus, ..., extraColumns=c(),
 			.retainSE=FALSE) {
-  sel <- c('MxComputeLoop1', 'CHR','BP','SNP','statusCode','catch1',
+  sel <- c('MxComputeLoop1', 'CHR','BP','SNP','A1','A2','statusCode','catch1',
 	   focus,paste0(focus,'SE'), extraColumns)
   got <- list()
   for (p1 in path) {
@@ -53,7 +55,7 @@ loadResults <- function(path, focus, ..., extraColumns=c(),
 #' tdir <- tempdir()
 #' dir <- system.file("extdata", package = "gwsem")
 #' pheno <- data.frame(anxiety=rnorm(500))
-#' m1 <- buildOneItem(pheno, 'anxiety')
+#' m1 <- buildItem(pheno, 'anxiety')
 #' GWAS(m1, file.path(dir,"example.pgen"),
 #'     file.path(tdir,"out.log"))
 #' got <- loadResults(file.path(tdir,"out.log"), "snp2anxiety")
